html {
  font-family: BlinkMacSystemFont, -apple-system, "Segoe UI", Roboto, Oxygen,
    Ubuntu, Cantarell, "Fira Sans", "Droid Sans", "Helvetica Neue", Helvetica,
    Arial, sans-serif;
  -webkit-font-smoothing: antialiased;
  background-color: #fff;
  font-size: 16px;
}
body {
  color: #4a4a4a;
  margin: 8px;
  font-size: 1em;
  font-weight: 400;
}
main {
  width: 100%;
  display: flex;
  flex-direction: column;
  align-items: center;
}
a {
  color: #3273dc;
  cursor: pointer;
  text-decoration: none;
}
a:hover {
  color: #000;
}
button {
  color: #fff;
  background-color: #3298dc;
  border-color: transparent;
  cursor: pointer;
  text-align: center;
}
button:hover {
  background-color: #2793da;
  flex: none;
}
.spacer {
  flex: auto;
}
.small {
  font-size: 0.75rem;
}
header {
  margin-top: 16px;
  display: flex;
  align-items: center;
}
.header-label {
  margin-right: 4px;
}
.benchmark-set {
  margin: 8px 0;
  width: 100%;
  display: flex;
  flex-direction: column;
}
.benchmark-title {
  font-size: 3rem;
  font-weight: 600;
  word-break: break-word;
  text-align: center;
}
.benchmark-graphs {
  display: flex;
  flex-direction: row;
  justify-content: space-around;
  align-items: center;
  flex-wrap: wrap;
  width: 100%;
}
.benchmark-chart {
  max-width: 1000px;
}

.benchmark-table {
  line-height: 1.5;
  color: #000 !important;
  box-sizing: inherit;
  font-size: 15px;
  margin-top: 20px;
  margin-bottom: 40px;
  border-collapse: collapse;
  width: 500px;
  padding: 8px;
  text-align: left;
  border-bottom: 1px solid #ddd;
}
